local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Click2Cast")
local T = AceLibrary("Tablet-2.0")

local actions = {
	menu = L["menu"],
	spell = L["spell"],
	target = L["target"],
	focus = L["focus"],
	assist = L["assist"],
	macro = L["macro"],
	none = L["none"],
}
local kbactions = {
	spell = L["spell"],
	macro = L["macroId"],
	none = L["none"],
}

local translatedModifiers = {
	SHIFT = L["Shift"],
	ALT = L["Alt"],
	CTRL = L["Ctrl"],
}

local niceModifier = setmetatable({}, {__index =
	function(self, key)
		if type(key) ~= "string" then return nil end
		local x = strsplit("-", key)
		local t = {}
		for i = 1, select("#", x) do
			local cur = (select(i, x))
			if not translatedModifiers[cur] then break end
			t[#t + 1] = translatedModifiers[cur]
		end
		if #t < 1 then return nil end
		self[key] = table.concat(t, "-")
		for i in ipairs(t) do t[i] = nil end
		return self[key]
	end
})

local keybindingOnly = {BUTTON1=true, BUTTON2=true, BUTTON3=true, BUTTON4=true, BUTTON5=true}

local function getOrder(binding)
	local num = 0
	if binding:find("ALT%-") then
		num = num + 3
	end
	if binding:find("CTRL%-") then
		num = num + 4
	end
	if binding:find("SHIFT%-") then
		num = num + 5
	end

	local button = tonumber(binding:match("BUTTON(%d)$"))
	if binding:find("BUTTON") then
		num = num + button * 8
	else
		num = 1
	end
	return num
end

local function createSubMenu(self, value)
	local name = dewdrop:EncodeKeybinding(value)
	local validateActions
	if value:find("BUTTON") then
		validateActions = actions
	else
		validateActions = kbactions
	end
	self.optionsTable.args[value] = {
		name = name or value.."bug",
		desc = name or value.."bug",
		order = getOrder(value),
		type = "group",
		args = {
			actions = {
				type = "text",
				name = L["Set Action"],
				desc = L["actions"],
				order = 1,
				validate = validateActions,
				get = function()
					return self.db.char.clickOptions[value].action
				end,
				set = function(option)
					self.db.char.clickOptions[value].action = option
					self:SetAllFrames()
					self:SetKeyboardBindings()
					if self.actionText[self.db.char.clickOptions[value].action] then
						self.optionsTable.args[value].args.action.name = L[option]
						self.optionsTable.args[value].args.action.desc = L[option.."Desc"]
					end
				end,
			},
			remove = {
				type = "execute",
				name = L["Remove"],
				desc = L["RemoveDesc"],
				func = function()
					self.db.char.clickOptions[value] = nil
					self.optionsTable.args[value] = nil
					self:DeleteAction(value)
					self:DisableKeyboardBindings(value)
				end,
			},
			action = {
				name = L["Action Text"],
				type = "text",
				desc = L["actiontext"],
				order = 2,
				get = function()
					return self.db.char.clickOptions[value].actiontext or ""
				end,
				usage = "",
				set = function(option)
					self.db.char.clickOptions[value].actiontext = option
					self:SetAllFrames()
					self:SetKeyboardBindings()
				end,
				hidden = function()
					return not (self.actionText[self.db.char.clickOptions[value].action])
				end,
			},
			rank = {
				name = L["Rank"],
				type = "text",
				desc = L["rankDesc"],
				order = 3,
				get = function()
					return self.db.char.clickOptions[value].rank or ""
				end,
				usage = "",
				set = function(option)
					self.db.char.clickOptions[value].rank = option:match("(%d+)")
					self:SetAllFrames()
					self:SetKeyboardBindings()
				end,
				hidden = function()
					return not (self.db.char.clickOptions[value].action == "spell")
				end,
			},
			withoutframes = {
				name = L["Outside Unitframes"],
				type = "toggle",
				desc = L["Mousebind works outside unit frames"],
				get = function()
					return self.db.char.clickOptions[value].outside
				end,
				set = function(option)
					self.db.char.clickOptions[value].outside = option
					self:SetAllFrames()
					self:SetKeyboardBindings()
				end,
				hidden = function()
					return not (self.db.char.clickOptions[value].action == "spell") or not value:find("BUTTON")
				end,
				order = 4,
			},
		}
	}
	if self.actionText[self.db.char.clickOptions[value].action] then
		self.optionsTable.args[value].args.action.name = L[self.db.char.clickOptions[value].action]
		self.optionsTable.args[value].args.action.desc = L[self.db.char.clickOptions[value].action.."Desc"]
	end
end

function Click2Cast:CreateMenu()
	self.optionsTable = {
		name = L["C2C"],
		desc = L["C2CDesc"],
		order = 3,
		handler = Click2Cast,
		type = "group",
		args = {},
		disabled = function()
			if InCombatLockdown() then
				return true
			end
		end,
	}
	self.optionsTable.args.new = {
		name = L["New Keybinding"],
		desc = L["New Keybinding for a action"],
		order = 200,
		type = "text",
		validate = "keybinding",
		set = function(value)
			if value and not self.optionsTable.args[value] then
				createSubMenu(self, value)
			end
		end,
		get = function()
			return nil
		end,
--		keybindingOnly = keybindingOnly,
	}
	for k in pairs(self.db.char.clickOptions) do
		createSubMenu(self, k)
	end

	Click2Cast:RegisterChatCommand("/Click2Cast", "/c2c", "/cc", self.optionsTable, "CLICK2CAST")
	Click2Cast.OnMenuRequest = self.optionsTable
end

local tmp = {}
local function mysort(alpha, bravo)
	if not alpha or not bravo then
		return false
	end
	return getOrder(alpha) < getOrder(bravo)
end

function Click2Cast:OnTooltipUpdate()
	local db = self.db.char.clickOptions
	local cat

	local showAction = {["spell"] = true,}
	for k,v in pairs(db) do
		table.insert(tmp, k)
	end
	table.sort(tmp, mysort)
	local last_button = nil
	for i,k in ipairs(tmp) do
		if k:find("BUTTON") then
			tmp[i] = nil
			local v = db[k]
			local button = tonumber(k:match("BUTTON(%d)$"))
			if last_button ~= button then
				last_button = button
				cat = T:AddCategory(
					"text", dewdrop:EncodeKeybinding("BUTTON" .. button),
					"columns", 2,
					"child_textR", 1,
					"child_textG", 1,
					"child_textB", 0,
					"child_textR2", 1,
					"child_textG2", 1,
					"child_textB2", 1
				)
			end
			local text
			if not k:find("%-") then
				text = L["No Modifier"]
			else
				text = niceModifier[k]
			end
			if v.actiontext and showAction[v.action] then
				cat:AddLine(
					"text", text,
					"text2", v.actiontext
				)
			else
				cat:AddLine(
					"text", text,
					"text2", L[v.action]
				)
			end
		end
	end

	T:SetHint(L["|cffeda55fRight-Click|r to configure clickcast actions."])
end

function Click2Cast:RecreateMenu()
	for k in pairs(self.optionsTable.args) do
		if k:find("^ALT.-BUTTON%d$") or k:find("^CTRL.-BUTTON%d$") or k:find("^SHIFT.-BUTTON%d$") or k:find("^BUTTON%d$") then
			self.optionsTable.args[k] = nil
		end
	end
	for k in pairs(self.db.char.clickOptions) do
		createSubMenu(self, k)
	end
end

